@@ -16,6 +16,7 @@ using System.IO;
 using System.Net;
 using System.Xml.Serialization;
 using System.Text.RegularExpressions;
+using System.Text;
 
 namespace XSDDiagram
 {
@@ -56,14 +57,7 @@ namespace XSDDiagram
 
         public void LoadSchema(string fileName)
         {
-            this.firstElement = null;
-            this.elements.Clear();
-            this.hashtableElementsByName.Clear();
-            this.hashtableElementsByName[""] = null;
-            this.hashtableAttributesByName.Clear();
-            this.hashtableAttributesByName[""] = null;
-            this.loadError.Clear();
-            this.listOfXsdFilename.Clear();
+            Cleanup();
 
             string url = fileName.Trim(), baseUrl = "";
             if (url.IndexOf("http://") == 0 || url.IndexOf("https://") == 0)
@@ -79,6 +73,23 @@ namespace XSDDiagram
             ImportSchema(fileName, baseUrl);
         }
 
+        public void Cleanup()
+        {
+            this.firstElement = null;
+            this.elements.Clear();
+            this.hashtableElementsByName.Clear();
+            this.hashtableElementsByName[""] = null;
+            this.hashtableAttributesByName.Clear();
+            this.hashtableAttributesByName[""] = null;
+            this.loadError.Clear();
+            this.listOfXsdFilename.Clear();
+        }
+
+        public bool IsLoaded()
+        {
+            return this.listOfXsdFilename.Count > 0;
+        }
+
         private void ImportSchema(string fileName, string baseUrl)
         {
             System.Diagnostics.Trace.WriteLine("ImportSchema: " + fileName);
@@ -249,6 +260,7 @@ namespace XSDDiagram
                 if (!File.Exists(loadedFileName))
                 {
                     WebClient webClient = new WebClient();
+                    webClient.Encoding = Encoding.UTF8;
                     bool tryAgain = false;
                     int attemptCount = 0;
                     do
@@ -453,17 +465,17 @@ namespace XSDDiagram
 
         void schemaSerializer_UnknownAttribute(object sender, XmlAttributeEventArgs e)
         {
-            this.loadError.Add("Unkonwn attribute (" + e.LineNumber + ", " + e.LinePosition + "): " + e.Attr.Name);
+            this.loadError.Add("Unknown attribute (" + e.LineNumber + ", " + e.LinePosition + "): " + e.Attr.Name);
         }
 
         void schemaSerializer_UnknownElement(object sender, XmlElementEventArgs e)
         {
-            this.loadError.Add("Unkonwn element (" + e.LineNumber + ", " + e.LinePosition + "): " + e.Element.Name);
+            this.loadError.Add("Unknown element (" + e.LineNumber + ", " + e.LinePosition + "): " + e.Element.Name);
         }
 
         void schemaSerializer_UnknownNode(object sender, XmlNodeEventArgs e)
         {
-            this.loadError.Add("Unkonwn node (" + e.LineNumber + ", " + e.LinePosition + "): " + e.Name);
+            this.loadError.Add("Unknown node (" + e.LineNumber + ", " + e.LinePosition + "): " + e.Name);
         }
 
         void schemaSerializer_UnreferencedObject(object sender, UnreferencedObjectEventArgs e)
