@@ -20,19 +20,26 @@ namespace XSDDiagram.Rendering
     {
         #region Private Fields
 
-        private bool  _showBoundingBox;
-		private Size  _size;
-		private Size  _padding;
+        private bool _showBoundingBox;
+        private bool _showDocumentation;
+        private Size _size;
+		private Size _padding;
         private float _scale;
+        private float _lastScale;
 
         private Font _font;
+        private Font _fontScaled;
         private Font _smallFont;
+        private Font _smallFontScaled;
+        private Font _documentationFont;
+        private Font _documentationFontScaled;
 
-		private Rectangle         _boundingBox;
+        private Rectangle _boundingBox;
 		private DiagramAlignement _alignement;
 
-        private List<DiagramItem> _rootElements;
         private IDictionary<string, XSDObject> _elementsByName;
+        private List<DiagramItem> _rootElements;
+        private DiagramItem _selectedElement;
 
         private XMLSchema.any     _fakeAny;
 
@@ -42,12 +49,14 @@ namespace XSDDiagram.Rendering
 
         public Diagram()
         {
-            _scale                   = 1.0f;
-            _size                    = new Size(100, 100);
-            _padding                 = new Size(10, 10);
-            _boundingBox             = Rectangle.Empty;
-            _alignement              = DiagramAlignement.Center;
-            _rootElements            = new List<DiagramItem>();
+            _scale = 1.0f;
+            _lastScale = 1.0f;
+            _size = new Size(100, 100);
+            _padding = new Size(10, 10);
+            _boundingBox = Rectangle.Empty;
+            _alignement = DiagramAlignement.Center;
+            _rootElements = new List<DiagramItem>();
+            _selectedElement = null;
             _elementsByName = new Dictionary<string, XSDObject>(StringComparer.OrdinalIgnoreCase);
         }
 
@@ -60,13 +69,19 @@ namespace XSDDiagram.Rendering
 		public Rectangle BoundingBox { get { return _boundingBox; } }
 		public float Scale { get { return _scale; } set { _scale = value; } }
 		public DiagramAlignement Alignement { get { return _alignement; } set { _alignement = value; } }
-		public bool ShowBoundingBox { get { return _showBoundingBox; } set { _showBoundingBox = value; } }
+        public bool ShowBoundingBox { get { return _showBoundingBox; } set { _showBoundingBox = value; } }
+        public bool ShowDocumentation { get { return _showDocumentation; } set { _showDocumentation = value; } }
 
-		public Font Font { get { return _font; } set { _font = value; } }
-		public Font SmallFont { get { return _smallFont; } set { _smallFont = value; } }
+        public Font Font { get { return _font; } set { _font = value; } }
+        public Font FontScaled { get { return _fontScaled; } set { _fontScaled = value; } }
+        public Font SmallFont { get { return _smallFont; } set { _smallFont = value; } }
+        public Font SmallFontScaled { get { return _smallFontScaled; } set { _smallFontScaled = value; } }
+        public Font DocumentationFont { get { return _documentationFont; } set { _documentationFont = value; } }
+        public Font DocumentationFontScaled { get { return _documentationFontScaled; } set { _documentationFontScaled = value; } }
 
         public IDictionary<string, XSDObject> ElementsByName { get { return _elementsByName; } set { _elementsByName = value; } }
 		public List<DiagramItem> RootElements { get { return _rootElements; } }
+        public DiagramItem SelectedElement { get { return _selectedElement; } }
 
         #endregion
 
@@ -327,6 +342,8 @@ namespace XSDDiagram.Rendering
 			{
 				DiagramItem childDiagramGroup = new DiagramItem();
 				childDiagramGroup.ItemType = DiagramItemType.group;
+
+				XMLSchema.group referenceGroup = null;
 				if (childGroup.@ref != null)
 				{
 					childDiagramGroup.IsReference = true;
@@ -337,7 +354,10 @@ namespace XSDDiagram.Rendering
                     {
                         XMLSchema.group group = grpObject.Tag as XMLSchema.group;
                         if (group != null)
+                        {
+                            referenceGroup = childGroup;
                             childGroup = group;
+                        }
                     }
 				}
 				else if (type == DiagramItemGroupType.Group)
@@ -353,13 +373,13 @@ namespace XSDDiagram.Rendering
 				childDiagramGroup.Diagram = this;
 				childDiagramGroup.TabSchema = childGroup;
 				int occurrence;
-				if (int.TryParse(childGroup.minOccurs, out occurrence))
+				if (int.TryParse(referenceGroup != null ? referenceGroup.minOccurs : childGroup.minOccurs, out occurrence))
 					childDiagramGroup.MinOccurrence = occurrence;
 				else
 					childDiagramGroup.MinOccurrence = -1;
 				//try { childDiagramGroup.MinOccurrence = int.Parse(childGroup.minOccurs); }
 				//catch { childDiagramGroup.MinOccurrence = -1; }
-				if (int.TryParse(childGroup.maxOccurs, out occurrence))
+				if (int.TryParse(referenceGroup != null ? referenceGroup.maxOccurs : childGroup.maxOccurs, out occurrence))
 					childDiagramGroup.MaxOccurrence = occurrence;
 				else
 					childDiagramGroup.MaxOccurrence = -1;
@@ -413,26 +433,35 @@ namespace XSDDiagram.Rendering
 
 		public void Clear()
 		{
-			_rootElements.Clear();
-		}
+            _elementsByName.Clear();
+            _rootElements.Clear();
+            _selectedElement = null;
+        }
 
-		public void Layout(Graphics g)
+        public void Layout(Graphics g)
 		{
  			string fontName = "Arial"; // "Verdana"; // "Arial";
 
-            if (_font != null)
+            if (_font == null)
             {
-                _font.Dispose();
+                _font = new Font(fontName, 10.0f, FontStyle.Bold, GraphicsUnit.Pixel);
+                _smallFont = new Font(fontName, 9.0f, GraphicsUnit.Pixel);
+                _documentationFont = new Font(fontName, 10.0f, GraphicsUnit.Pixel);
             }
-            if (_smallFont != null)
+            if (_fontScaled == null || _lastScale != _scale)
             {
-                _smallFont.Dispose();
+                //float fontScale = (float)Math.Pow(_scale, 2.0);
+                float fontScale = _scale;
+                if (_fontScaled != null) _fontScaled.Dispose();
+                _fontScaled = new Font(fontName, 10.0f * fontScale, FontStyle.Bold, GraphicsUnit.Pixel);
+                if (_smallFontScaled != null) _smallFontScaled.Dispose();
+                _smallFontScaled = new Font(fontName, 9.0f * fontScale, GraphicsUnit.Pixel);
+                if (_documentationFontScaled != null) _documentationFontScaled.Dispose();
+                _documentationFontScaled = new Font(fontName, 10.0f * fontScale, GraphicsUnit.Pixel);
+                _lastScale = _scale;
             }
 
-			_font      = new Font(fontName, 10.0f * (float)Math.Pow(_scale, 2.0), FontStyle.Bold, GraphicsUnit.Pixel);
-			_smallFont = new Font(fontName, 9.0f * (float)Math.Pow(_scale, 2.0), GraphicsUnit.Pixel);
-
-			foreach (DiagramItem element in _rootElements)
+            foreach (DiagramItem element in _rootElements)
 				element.GenerateMeasure(g);
 
 			_boundingBox = new Rectangle(0, 0, 100, 0);
@@ -587,6 +616,17 @@ namespace XSDDiagram.Rendering
 			}
         }
 
+        public void SelectElement(DiagramItem element)
+        {
+            if(_selectedElement != null)
+                _selectedElement.IsSelected = false;
+            if (element != null)
+            {
+                _selectedElement = element;
+                element.IsSelected = true;
+            }
+        }
+
         #endregion
 
         #region Private Methods
@@ -765,7 +805,16 @@ namespace XSDDiagram.Rendering
 								if (diagramCompositors != null)
 									ExpandChildren(diagramCompositors);
 							}
-						}
+
+                            XMLSchema.group groupAll = extensionType.all as XMLSchema.group;
+                            if (groupAll != null)
+                            {
+                                DiagramItem diagramCompositors = AddCompositors(parentDiagramElement, groupAll, DiagramItemGroupType.All, extensionType.@base.Namespace);
+                                parentDiagramElement.ShowChildElements = true;
+                                if (diagramCompositors != null)
+                                    ExpandChildren(diagramCompositors);
+                            }
+                        }
 						else if (complexContent.Item is XMLSchema.restrictionType)
 						{
 							XMLSchema.restrictionType restrictionType = complexContent.Item as XMLSchema.restrictionType;
