@@ -14,6 +14,7 @@ using System;
 using System.Drawing;
 using System.Drawing.Imaging;
 using System.Drawing.Drawing2D;
+using System.Text.RegularExpressions;
 
 namespace XSDDiagram.Rendering
 {
@@ -55,6 +56,10 @@ namespace XSDDiagram.Rendering
             {
                 return _graphics;
             }
+            set
+            {
+                _graphics = value;
+            }
         }
 
         #endregion
@@ -115,11 +120,16 @@ namespace XSDDiagram.Rendering
         {
             //System.Diagnostics.Trace.WriteLine("DiagramElement.Paint\n\tName: " + drawingItem.Name);
 
-            Brush background = new SolidBrush(Color.White);
+            Color backgroundColor = Color.White;
+            if(drawingItem.IsSelected)
+                backgroundColor = Color.FromArgb(0xA6, 0xCA, 0xF0);
+
+            Brush background = new SolidBrush(backgroundColor);
+            Brush backgroundExpandBox = new SolidBrush(Color.White);
             SolidBrush foreground = new SolidBrush(Color.Black);
             if (drawingItem.IsDisabled)
             {
-                background = new HatchBrush(HatchStyle.BackwardDiagonal, Color.Gray, Color.White);
+                background = new HatchBrush(HatchStyle.BackwardDiagonal, Color.Gray, backgroundColor);
                 foreground = new SolidBrush(Color.Gray);
             }
             Pen foregroundPen = new Pen(foreground);
@@ -158,18 +168,19 @@ namespace XSDDiagram.Rendering
             Rectangle scaledElementBox = drawingItem.ScaleRectangle(drawingItem.ElementBox);
 
             // Draw the children lines
-            if (drawingItem.ShowChildElements)
+            if (drawingItem.ShowChildElements && drawingItem.ChildElements.Count > 0)
             {
                 Pen foregroundInheritPen = new Pen(foreground);
                 foregroundInheritPen.StartCap = LineCap.Round;
                 foregroundInheritPen.EndCap = LineCap.Round;
 
-                if (drawingItem.ChildElements.Count == 1)
+                bool showDocumentation = (drawingItem.Diagram.ShowDocumentation && drawingItem.DocumentationBox != null);
+                if (drawingItem.ChildElements.Count == 1 && !showDocumentation)
                 {
                     int parentMidleY = drawingItem.ScaleInt(drawingItem.Location.Y + drawingItem.Size.Height / 2);
                     _graphics.DrawLine(foregroundInheritPen, drawingItem.ScaleInt(drawingItem.Location.X + drawingItem.Size.Width), parentMidleY, drawingItem.ScaleInt(drawingItem.ChildElements[0].Location.X), parentMidleY);
                 }
-                else if (drawingItem.ChildElements.Count > 1)
+                else if (drawingItem.ChildElements.Count > 1 || showDocumentation)
                 {
                     DiagramItem firstElement = drawingItem.ChildElements[0];
                     DiagramItem lastElement = drawingItem.ChildElements[drawingItem.ChildElements.Count - 1];
@@ -481,7 +492,32 @@ namespace XSDDiagram.Rendering
                 stringFormatText.Alignment = StringAlignment.Center;
                 stringFormatText.LineAlignment = StringAlignment.Center;
                 stringFormatText.FormatFlags |= StringFormatFlags.NoClip; //MONOFIX
-                _graphics.DrawString(drawingItem.Name, drawingItem.Font, foreground, new RectangleF(scaledElementBox.X, scaledElementBox.Y, scaledElementBox.Width, scaledElementBox.Height), stringFormatText);
+                _graphics.DrawString(drawingItem.Name, drawingItem.FontScaled, foreground, new RectangleF(scaledElementBox.X, scaledElementBox.Y, scaledElementBox.Width, scaledElementBox.Height), stringFormatText);
+            }
+
+            // Draw Documentation
+            if (drawingItem.Diagram.ShowDocumentation && drawingItem.DocumentationBox != null)
+            {
+                string text = drawingItem.GetTextDocumentation();
+                if (text != null)
+                {
+                    StringFormat stringFormatText = new StringFormat();
+                    stringFormatText.Alignment = StringAlignment.Near;
+                    stringFormatText.LineAlignment = StringAlignment.Near;
+                    stringFormatText.Trimming = StringTrimming.EllipsisCharacter;
+                    stringFormatText.FormatFlags |= StringFormatFlags.NoClip; //MONOFIX
+
+                    Rectangle scaledDocumentationBox = drawingItem.ScaleRectangle(drawingItem.DocumentationBox);
+                    if (drawingItem.Diagram.ShowBoundingBox)
+                    {
+                        int color = 255 - drawingItem.Depth * 8;
+                        _graphics.FillRectangle(new SolidBrush(Color.FromArgb(color, 255 - color, color)), scaledDocumentationBox);
+                        _graphics.DrawRectangle(foregroundPen, scaledDocumentationBox);
+                    }
+                    _graphics.DrawString(text, drawingItem.DocumentationFontScaled, foreground
+                        , new RectangleF(scaledDocumentationBox.X, scaledDocumentationBox.Y, scaledDocumentationBox.Width, scaledDocumentationBox.Height)
+                        , stringFormatText);
+                }
             }
 
             // Draw occurences small text
@@ -496,7 +532,7 @@ namespace XSDDiagram.Rendering
                 PointF pointOccurences = new PointF();
                 pointOccurences.X = drawingItem.Diagram.Scale * (drawingItem.Location.X + drawingItem.Size.Width - 10);
                 pointOccurences.Y = drawingItem.Diagram.Scale * (drawingItem.Location.Y + drawingItem.Size.Height + 10);
-                _graphics.DrawString(occurences, drawingItem.SmallFont, foreground, pointOccurences, stringFormatOccurences);
+                _graphics.DrawString(occurences, drawingItem.SmallFontScaled, foreground, pointOccurences, stringFormatOccurences);
             }
 
             // Draw type
@@ -543,7 +579,7 @@ namespace XSDDiagram.Rendering
             if (drawingItem.HasChildElements)
             {
                 Rectangle scaledChildExpandButtonBox = drawingItem.ScaleRectangle(drawingItem.ChildExpandButtonBox);
-                _graphics.FillRectangle(background, scaledChildExpandButtonBox);
+                _graphics.FillRectangle(backgroundExpandBox, scaledChildExpandButtonBox);
                 _graphics.DrawRectangle(foregroundPen, scaledChildExpandButtonBox);
 
                 Point middle = new Point(scaledChildExpandButtonBox.Width / 2, scaledChildExpandButtonBox.Height / 2);
